JSB.newAddon = function (mainPath) {
  JSB.require('DataImporter');
  let newAddonClass = JSB.defineClass('WordEx : JSExtension', /*Instance members*/{
    sceneWillConnect: function () {
      self.webController = WebViewController.new();
    },
    sceneDidDisconnect: function () {
    },
    sceneWillResignActive: function () {
    },
    sceneDidBecomeActive: function () {
    },
    notebookWillOpen: function (notebookid) {
      NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onPopupMenuOnNote:', 'PopupMenuOnNote');
      NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onProcessExcerptText:', 'ProcessNewExcerpt');
      NSNotificationCenter.defaultCenter().addObserverSelectorName(self, 'onProcessExcerptText:', 'ChangeExcerptRange');
      self.status = NSUserDefaults.standardUserDefaults().objectForKey('marginnote_wordex');
      self.data = DataImporter.new().importData()
    },
    notebookWillClose: function (notebookid) {
      NSNotificationCenter.defaultCenter().removeObserverName(self, 'PopupMenuOnNote');
      NSNotificationCenter.defaultCenter().removeObserverName(self, 'ProcessNewExcerpt');
      NSNotificationCenter.defaultCenter().removeObserverName(self, 'ChangeExcerptRange');
    },
    documentDidOpen: function (docmd5) {
    },
    documentWillClose: function (docmd5) {
    },
    controllerWillLayoutSubviews: function (controller) {
    },
    queryAddonCommandStatus: function () {
      if (Application.sharedInstance().studyController(self.window).studyMode < 3)
        return {
          image: 'wordex.png',
          object: self,
          selector: 'toggleWordEx:',
          checked: (self.status ? true : false)
        };
      return null;
    },

    onPopupMenuOnNote: function (sender) {
      if (!Application.sharedInstance().checkNotifySenderInWindow(sender, self.window)) return;
      if (!self.status) return;
      let note = sender.userInfo.note;
      if (note) {
        let timerCount = 0;
        NSTimer.scheduledTimerWithTimeInterval(1, true, function (timer) {
          let title = note.noteTitle.trim()
          if (title) {
            title = title.toLowerCase()
            tmp = title.split(";")[0]
            if (self.data[tmp] && title != self.data[tmp]) {
              UndoManager.sharedInstance().undoGrouping('WordEx', note.notebookId, function () {
                note.noteTitle = self.data[tmp]
                Database.sharedInstance().setNotebookSyncDirty(note.notebookId);
              });
              NSNotificationCenter.defaultCenter().postNotificationNameObjectUserInfo('RefreshAfterDBChange', self, { topicid: note.notebookId });
            }
          }
          timerCount++;
          if (timerCount >= 1) {
            timer.invalidate();
          }
        });
      }
    },

    onProcessExcerptText: function (sender) {
      if (!Application.sharedInstance().checkNotifySenderInWindow(sender, self.window)) return;
      if (!self.status) return;
      let noteid = sender.userInfo.noteid;
      let note = Database.sharedInstance().getNoteById(noteid);
      // 没有多个摘录，这个 groupid 只有超过一个摘录后才有，而且是属于这个摘录的，而点击卡片默认点击第一个摘录，所有点击卡片这个 groupid 无效
      if (note && note.excerptText && !note.groupNoteId) {
        let timerCount = 0;
        NSTimer.scheduledTimerWithTimeInterval(1, true, function (timer) {
          let text = note.excerptText.trim()
          // 新建卡片的时候，而不是有卡片了摘录
          if (text && text.length < 250) {
            // 没有匹配到，直接将摘录赋给标题
            UndoManager.sharedInstance().undoGrouping('WordEx', note.notebookId, function () {
              if (!self.data[text.toLowerCase()]) {
                note.noteTitle = text
                note.excerptText = ""
              }
              else {
                note.noteTitle = self.data[text.toLowerCase()]
                note.excerptText = ""
              }
              Database.sharedInstance().setNotebookSyncDirty(note.notebookId);
            });
            NSNotificationCenter.defaultCenter().postNotificationNameObjectUserInfo('RefreshAfterDBChange', self, { topicid: note.notebookId });
          }
          timerCount++;
          if (timerCount >= 2) {
            timer.invalidate();
          }
        });
      }
    },

    toggleWordEx: function (sender) {
      if (self.status) {
        self.status = false;
      }
      else {
        self.status = true;
        Application.sharedInstance().showHUD('开启单词词形自动补全&转换为标题', self.window, 2);
      }
      NSUserDefaults.standardUserDefaults().setObjectForKey(self.status, 'marginnote_wordex');
      Application.sharedInstance().studyController(self.window).refreshAddonCommands();
    },
  }, /*Class members*/{
    addonDidConnect: function () {
    },
    addonWillDisconnect: function () {
    },
    applicationWillEnterForeground: function () {
    },
    applicationDidEnterBackground: function () {
    },
    applicationDidReceiveLocalNotification: function (notify) {
    },
  });
  return newAddonClass;
};